#! /usr/bin/python

"""Example of Missionaries and Cannibals for use with AIMA code"""

from search import *
import sys

class MC(Problem):
    """ Classic missionaries and Cannibals problem.  A state is
     represented by a three typle (M,C,B) where M, C and B are the
     number of missionaries, cannibals and boats on the left bank,
     repsectively.  A state is legal if it is not the case that there
     are missionaries on a bank that re outnumbered by canniabls.
     Actions are that one or two people can row the boat from one bank
     to the other.  The problem is to find a sequence of actions to go
     from (3,3,1) to (0,0,0)."""

    def __init__(self, initial=pass, goal=pass):
        pass

    def __repr__(self):
        """returns a string representing the object"""
        pass

    def goal_test (self, state):
        """returns true if state is a goal state"""
        pass

    def successor(self, state):
        """returns a list of successors to state"""
        pass

def solutions (M_max, C_max):

    """ Prints an array showing MC problems that have solutions for
    initial states with 0..M_max and 0..C_max millionaries and
    canibals, repectively.  We use breadth_first_graph_search (from
    search.py) to test for a solution.  It returns a goal node if it
    finds a solution and None if not."""
    
    # print column headers
    print 'c:', 
    for c in range(C_max + 1):
        print c,
    print
    print ' +' + ('-' * (2* (C_max + 1)))

    # print rows
    for m in range(M_max + 1):
        # print row header
        print str(m)+'|',
        # print a row
        for c in range(C_max + 1):
            #
            if breadth_first_graph_search(MC(pass))
                print"1",
            else:
                print "0",
        print


def main(initial=None, goal=None):
    """ Called if the file is invoked from the command line.  Initial
    and goal parameters can be optionally given on the command line."""

    # Search algorithms (from search.py) to test
    searchers = [breadth_first_tree_search, breadth_first_graph_search, depth_first_graph_search, \
                 iterative_deepening_search, depth_limited_search]

    # create a list of search problems using the ptional args
    if initial and goal:
        problems =[MC(initial=initial, goal=goal)]
    elif initial:
        problems =[MC(initial=initial)]
    else:        
        problems =[MC()]

    # apply the 
    for p in problems:
        for s in searchers:
            solution = s(p)
            if solution:
                print "Solution to %s found by %s" % (p, s.__name__)
                path = solution.path()
                path.reverse()
                print path
            else:
                print "No solution to %s found by %s" % (p, s.__name__)
            print
    print "SUMMARY: successors/goal tests/states generated/solution"
    compare_searchers(problems=problems, header=['', ''], searchers=searchers)

# if called from the command line, invoke main using the optional
# arguments for the intial and goal states if provided

if __name__ == "__main__":

    # 1st command line arg is always the command name (e.g., my.py)
    nargs = len(sys.argv)

    # Note that we call eval on any command line arguments to convert
    # the string into a python data structure

    if nargs > 3:
        print "Usage: python mc.py [initial_state [goal_state]]"
    elif nargs == 3:
        main(initial=eval(sys.argv[1]), goal=eval(sys.argv[2]))
    elif nargs == 2:
        main(initial=eval(sys.argv[1]))
    else:
        main()
