#!/usr/bin/python
# http://mathworld.wolfram.com/QueensProblem.html

import constraint
import random
import sys

def main():

    problem = constraint.Problem()

    # one variable for each column, call them 0..8
    variables = range(8)

    # each variable has same domain: a row number in 0..8
    domain = range(8)

    # add variables 0..8 each with domain 0..8
    problem.addVariables(variables, domain)

    # add constraints between pairs of variables: no two queens can be
    # on the same row or same diagonal

    for v1 in variables:
        for v2 in variables:
            # Don't add constraint between a variable and itself; only
            # add one constraint for a given pair disregarding order
            if v1 < v2:
                # Create function F that takes two arguments (the
                # candidate values) and will default col1 and col2 to
                # the variable's column numbers (i.e., v1 and v2)
                F = lambda row1, row2, col1=v1, col2=v2: \
                    row1 != row2 and abs(row1-row2) != abs(col1-col2) 

                # add constraint function F between varibles v1 and v2
                problem.addConstraint(F, (v1, v2))

    solutions = problem.getSolutions()

    if solutions:
        print "Found %d solutions, here is one\n" % len(solutions)
        showSolution(random.choice(solutions))
    else:
        print "Found no solutions :("

def showSolution(solution):
    # print solution as a board
    sys.stdout.write("   %s \n" % ("-"*31))
    for i in range(8):
        sys.stdout.write("  |")
        for j in range(8):
            if solution[j] == i:
                sys.stdout.write(" %d |" % j)
            else:
                sys.stdout.write("   |")
        sys.stdout.write("\n")
        if i != 7:
            sys.stdout.write("  |%s|\n" % ("-"*31))
    sys.stdout.write("   %s \n" % ("-"*31))

if __name__ == "__main__":
    main()

